/**
 * Linear Slider PACKAGED v:1.0
 * Glitch postproc fragment.
 * @author Tibi - FWDesign [https://webdesign-flash.ro/]
 * Copyright © Since 2006 All Rights Reserved.
 */

uniform sampler2D tDiffuse;
uniform vec2 uResolution;
uniform float uTime;
varying vec2 vUv;

float random(vec2 c) {
    return fract(sin(dot(c.xy, vec2(12.9898, 78.233))) * 43758.5453);
}

vec3 mod289(vec3 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}
vec4 mod289(vec4 x) {
    return x - floor(x * (1.0 / 289.0)) * 289.0;
}
vec4 permute(vec4 x) {
    return mod289(((x*34.0)+1.0)*x);
}
vec4 taylorInvSqrt(vec4 r) {
    return 1.79284291400159 - 0.85373472095314 * r;
}
float snoise(vec3 v) {
    const vec2  C = vec2(1.0/6.0, 1.0/3.0) ;
    const vec4  D = vec4(0.0, 0.5, 1.0, 2.0);
    
    // First corner
    
    vec3 i = floor(v + dot(v, C.yyy) );
    vec3 x0 = v - i + dot(i, C.xxx) ;
    
    // Other corners
    
    vec3 g_0 = step(x0.yzx, x0.xyz);
    vec3 l = 1.0 - g_0;
    vec3 i1 = min( g_0.xyz, l.zxy );
    vec3 i2 = max( g_0.xyz, l.zxy );
    
    //   x0 = x0 - 0.0 + 0.0 * C.xxx;
    
    //   x1 = x0 - i1  + 1.0 * C.xxx;
    //   x2 = x0 - i2  + 2.0 * C.xxx;
    //   x3 = x0 - 1.0 + 3.0 * C.xxx;
    vec3 x1 = x0 - i1 + C.xxx;
    vec3 x2 = x0 - i2 + C.yyy; // 2.0*C.x = 1/3 = C.y
    
    vec3 x3 = x0 - D.yyy;      // -1.0+3.0*C.x = -0.5 = -D.y
    
    
    // Permutations
    i = mod289(i);
    vec4 p = permute( permute( permute(
    i.z + vec4(0.0, i1.z, i2.z, 1.0 ))
    + i.y + vec4(0.0, i1.y, i2.y, 1.0 ))
    + i.x + vec4(0.0, i1.x, i2.x, 1.0 ));
    
    // Gradients: 7x7 points over a square, mapped onto an octahedron.
    
    // The ring size 17*17 = 289 is close to a multiple of 49 (49*6 = 294)
    float n_ = 0.142857142857; // 1.0/7.0
    
    vec3  ns = n_ * D.wyz - D.xzx;
    vec4 j = p - 49.0 * floor(p * ns.z * ns.z);  //  mod(p, 7*7)
    
    
    vec4 x_ = floor(j * ns.z);
    vec4 y_ = floor(j - 7.0 * x_ );    // mod(j, N)
    
    
    vec4 x = x_ *ns.x + ns.yyyy;
    vec4 y = y_ *ns.x + ns.yyyy;
    vec4 h = 1.0 - abs(x) - abs(y);
    vec4 b0 = vec4( x.xy, y.xy );
    vec4 b1 = vec4( x.zw, y.zw );
    
    //vec4 s0 = vec4(lessThan(b0, 0.0))*2.0 - 1.0;
    
    //vec4 s1 = vec4(lessThan(b1, 0.0))*2.0 - 1.0;
    vec4 s0 = floor(b0)*2.0 + 1.0;
    vec4 s1 = floor(b1)*2.0 + 1.0;
    vec4 sh = -step(h, vec4(0.0));
    vec4 a0 = b0.xzyw + s0.xzyw*sh.xxyy ;
    vec4 a1 = b1.xzyw + s1.xzyw*sh.zzww ;
    vec3 p0 = vec3(a0.xy, h.x);
    vec3 p1 = vec3(a0.zw, h.y);
    vec3 p2 = vec3(a1.xy, h.z);
    vec3 p3 = vec3(a1.zw, h.w);
    
    //Normalise gradients
    
    vec4 norm = taylorInvSqrt(vec4(dot(p0, p0), dot(p1, p1), dot(p2, p2), dot(p3, p3)));
    p0 *= norm.x;
    p1 *= norm.y;
    p2 *= norm.z;
    p3 *= norm.w;
    
    // Mix final noise value
    
    vec4 m = max(0.6 - vec4(dot(x0, x0), dot(x1, x1), dot(x2, x2), dot(x3, x3)), 0.0);
    m = m * m;
    return 42.0 * dot( m*m, vec4( dot(p0, x0), dot(p1, x1), dot(p2, x2), dot(p3, x3) ) );
}

const float interval = 3.0;
uniform float uBlocksStrength;

void main() {
    float strength = 0.0;

    vec2 shake = vec2(strength * 8.0 + 0.5) * vec2(
        random(vec2(uTime)) * 2.0 - 1.0, random(vec2(uTime * 2.0)) * 2.0 - 1.0
    ) / uResolution;

    float y = vUv.y * uResolution.y;
    float rgbWave = (
        snoise(vec3(0.0, y * 0.01, uTime * 400.0)) * (2.0 + strength * 32.0)
        * snoise(vec3(0.0, y * 0.02, uTime * 200.0)) * (1.0 + strength * 4.0)
        + step(0.9995, sin(y * 0.005 + uTime * 1.6)) * 12.0
        + step(0.9999, sin(y * 0.005 + uTime * 2.0)) * -18.0
    ) / uResolution.x;

    float rgbDiff = (uBlocksStrength * 0.5 * 6.0 + sin(uTime * 500.0 * + vUv.y * 40.0) * (20.0 * strength * uBlocksStrength + 1.0)) / uResolution.x;

    float rgbUvX = vUv.x + rgbWave;
    

    // Sample color with shake effect
    float r = texture2D(tDiffuse, vec2(rgbUvX + rgbDiff, vUv.y) + shake).r;
    float g = texture2D(tDiffuse, vec2(rgbUvX, vUv.y) + shake).g;
    float b = texture2D(tDiffuse, vec2(rgbUvX - rgbDiff, vUv.y) + shake).b;


    // Noise for white noise effect
    float whiteNoise = (random(vUv + mod(uTime, 10.0)) * 2.0 - 1.0) * (0.15 + strength * 0.15);


    // Block Noise
    float bnuTIme = floor(uTime * 20.0) * 200.0;
    float noiseX = step((snoise(vec3(0.0, vUv.x * 3.0, bnuTIme)) + 1.0) / 2.0, 0.12 + strength * 0.3);
    float noiseY = step((snoise(vec3(0.0, vUv.y * 3.0, bnuTIme)) + 1.0) / 2.0, 0.12 + strength * 0.3);
    float bnMask = noiseX * noiseY;

    float bnUvX = vUv.x + sin(bnuTIme) * 0.2 + rgbWave;

    float bnR = texture2D(tDiffuse, vec2(bnUvX + rgbDiff, vUv.y)).r * bnMask;
    float bnG = texture2D(tDiffuse, vec2(bnUvX, vUv.y)).g * bnMask;
    float bnB = texture2D(tDiffuse, vec2(bnUvX - rgbDiff, vUv.y)).b * bnMask;

    vec4 blockNoise = vec4(bnR, bnG, bnB, 1.0);


    // Second layer of block noise for added detail
    float bnuTIme2 = floor(uTime * 25.0) * 300.0;
    float noiseX2 = step((snoise(vec3(0.0, vUv.x * 2.0, bnuTIme2)) + 1.0) / 2.0, 0.12 * uBlocksStrength + strength * 0.5);
    float noiseY2 = step((snoise(vec3(0.0, vUv.y * 8.0, bnuTIme2)) + 1.0) / 2.0, 0.12 * uBlocksStrength + strength * 0.3);
    float bnMask2 = noiseX2 * noiseY2;

    float bnR2 = texture2D(tDiffuse, vec2(bnUvX + rgbDiff, vUv.y)).r * bnMask2;
    float bnG2 = texture2D(tDiffuse, vec2(bnUvX, vUv.y)).g * bnMask2;
    float bnB2 = texture2D(tDiffuse, vec2(bnUvX - rgbDiff, vUv.y)).b * bnMask2;
    vec4 blockNoise2 = vec4(bnR2, bnG2, bnB2, 1.0);


    // Wave noise effect
    float waveNoise = (sin(vUv.y * 1200.0) + 1.0) / 2.0 * (0.15 + strength * 0.2);


    // Blend the final output, preserving transparency
    vec4 finalColor = vec4(r, g, b, 1.0) * (1.0 - bnMask - bnMask2) + (whiteNoise + blockNoise + blockNoise2 - waveNoise);


    // Use the alpha from the original texture to ensure transparency is respected
    float textureAlpha = texture2D(tDiffuse, vUv).a;


    // Set the final color with alpha blending
    gl_FragColor = vec4(finalColor.rgb, finalColor.a * textureAlpha);
}
